#include "printf.h"
#include "as31glue.h"
#include "parse.h"	// for event codes
#include "string.h"
#include "stricmp.h"
#include "sta013.h"
#include "main.h"
#include "rand.h"

#include "display.h"

#define VOL_TIMEOUT 50
#define SCREEN_TIMEOUT 80

typedef enum {
   SCREEN1,
   SCREEN2,
   SCREEN3,
   SCREEN_MAX = 2
} screen_t;

typedef enum {
   CONTROL0,
   CONTROL1,
   CONTROL2,
   CONTROL3,
   CONTROL4,
   CONTROL5,
   CONTROL6,
   CONTROL7,
   CONTROL8,
   CONTROL9,
   CONTROL_MAX = 9
} control_t;

static code unsigned char last_control[SCREEN_MAX+1] = {CONTROL4,
							CONTROL8,
							CONTROL4};

static code unsigned char play_mode_description[MODE_MAX+1][11] = {
   " Dir Only ",
   " Dir Rand ",
   " Seqntial ",
   " Plst Rand",
   " All Rand "};

#define DISPLAY_WIDTH 24
#define FILEBUF_MAX 256
#define PLAYBUF_MAX 256

xdata unsigned char fxwrap;		// Position inside filebuf
xdata unsigned char fxscroll;	// Scroll ?
xdata unsigned char filebuf[FILEBUF_MAX];
xdata unsigned char fdisplaybuf[DISPLAY_WIDTH + 1];
xdata unsigned char filebuf_len;

xdata unsigned char pxwrap;               // Position inside playbuf
static bit pxscroll;	        // Scroll ?
xdata unsigned char playbuf[PLAYBUF_MAX];
xdata unsigned char pdisplaybuf[DISPLAY_WIDTH + 1];
xdata unsigned char playbuf_len;

xdata struct id3_tag_struct id3_tag;

// TODO: Remove this once display bug is fixed
xdata update_counter;

screen_t current_screen;
control_t current_control;

// display version
xdata unsigned char lcd_version;

//
// Timer usage:
//
// timer 0 - scrolling song name
// timer 1 - screen display timeout
// timer 2 - delays writing of Flash values until the user is finished tinkering with the controls
// timer 3
//


//
//	lcd_display_filename
//
//	Takes long file name.  Strips last 4 characters if
//	they are ".m3u".  Pads with a few spaces and
//	fills to DISPLAY_WIDTH characters if shorter than that.
//
void lcd_display_filename(const xdata unsigned char *filename)
{

   xdata unsigned char* source;
   xdata unsigned char* dest;

   // Kill any pending timer0 interrupts
   clear_timer(0);

   // make local copy (filename will be invalid on exit)
   // check length & truncate.  convert '_' to ' '.
   source = filename;
   dest = filebuf;
   filebuf_len = 0;
   while(*source != '\0')
   {
      filebuf_len++;
      if (filebuf_len >= FILEBUF_MAX)
      {
	 break;
      }
      else if (*source == '_')
      {
	 *dest = ' ';
      }
      else
      {
	 *dest = *source;
      }

      // this is a workaround for an sdcc bug (submitted 20011208)
      dest = dest;
      
      source++;
      dest++;
   }
   *dest = '\0';

   // .mp3 and .MP3
   if (filebuf_len > 4 && str_is_dot_mp3(filebuf + filebuf_len - 4))
   {
      filebuf_len -= 4;
      *(filebuf + filebuf_len) = '\0';
   }
	
   if (filebuf_len <= DISPLAY_WIDTH)
   {
      // pad with maximum number of spaces
      memset(filebuf + filebuf_len, ' ', DISPLAY_WIDTH);
      filebuf[DISPLAY_WIDTH] = '\0';
      fxscroll = 0;
   }
   else
   {
      // init scroll buffer
      memcpy(fdisplaybuf, filebuf, DISPLAY_WIDTH);
      fdisplaybuf[DISPLAY_WIDTH] = '\0';
      fxscroll = 1;
   }

   if (lcd_version == 0)
   {
      fxwrap = 23;
      lcd_update_2nd_line(0);
   }
   else
   {
      if (fxscroll == 0)
      {
	 // scrolling off and force display of new string
	 print("\\K2!0");
	 lcd_update_2nd_line(1);
      }
      else
      {
	 // just fill scrolling buffer and let the display do the work
	 printf("\\[\\K3!1%s    \\K3!0\\K2!1\\]", filebuf);
      }
   }
}






//
//	lcd_update_2nd_line
//
//	Called when timer 0 expires.
//	Displays fdisplaybuf on the second line of the display, then
//	scrolls fdisplaybuf to the left, and grabs the next
//	character from filebuf.
//
void lcd_update_2nd_line(const unsigned char refresh)
{
   xdata unsigned char font;

   if (current_control == CONTROL3)
      font = 33;  // bold
   else
      font = 32;  // normal
	
   if (lcd_version == 0)
   {
      if (fxscroll == 0)
      {
	 // not scrolling so just display it
	 printf("\\[\\B !\\C%c%s\\C \\]\r\n", (char) font, filebuf);
      }
      else
      {
	 printf("\\[\\B !\\C%c%s\\C \\]\r\n", (char) font, fdisplaybuf);

	 // shift buffer & insert new char
	 fxwrap++;
	 memcpy(fdisplaybuf, fdisplaybuf+1, DISPLAY_WIDTH - 1);

	 if(fxwrap < filebuf_len) {
	    fdisplaybuf[DISPLAY_WIDTH-1] = filebuf[fxwrap];
	 } else if (fxwrap < filebuf_len + 4) {
	    fdisplaybuf[DISPLAY_WIDTH-1] = ' ';
	 } else {
	    fxwrap = 0;
	    fdisplaybuf[DISPLAY_WIDTH-1] = filebuf[0];
	 }
      }

      // TODO: Remove this once display bug is fixed
      if((update_counter++)>10)
      {
	 update_counter=0;
	 lcd_change_menu(0);
      }
   }
   else
   {
      // ensure font is correct upon refresh (selection might have changed)
      if (refresh != 0)
      {
	 if (fxscroll != 0) 
	 {
	    printf("\\K1!%c", (char) font);
	 }
	 else
	 {
	    printf("\\[\\B !\\C%c%s\\C \\]\r\n", (char) font, filebuf);
	 }
      }
   }
}

//
//	lcd_display_playlist_name
//
//	Takes long file name.  Strips last 4 characters if
//	they are ".m3u".  Pads with a few spaces and
//	fills to DISPLAY_WIDTH characters if shorter than that.
//
void lcd_display_playlist_name(const xdata unsigned char *pl_name)
{

   xdata unsigned char* source;
   xdata unsigned char* dest;

   // Kill any pending timer0 interrupts
   clear_timer(0);

   // make local copy (pl_name will be invalid on exit)
   // check length & truncate.  convert '_' to ' '.
   source = pl_name;
   dest = playbuf;
   playbuf_len = 0;
   while(*source != '\0')
   {
      playbuf_len++;
      if (playbuf_len >= PLAYBUF_MAX)
      {
	 break;
      }
      else if (*source == '_')
      {
	 *dest = ' ';
      }
      else
      {
	 *dest = *source;
      }

      // this is a workaround for an sdcc bug (submitted 20011208)
      dest = dest;
      
      source++;
      dest++;
   }
   *dest = '\0';

   // .m3u and .M3U
   if (playbuf_len > 4 && str_is_dot_m3u(playbuf + playbuf_len - 4))
   {
      playbuf_len -= 4;
      *(playbuf + playbuf_len) = '\0';
   }
	
   if (playbuf_len <= DISPLAY_WIDTH)
   {
      // pad with maximum number of spaces
      memset(playbuf + playbuf_len, ' ', DISPLAY_WIDTH);
      playbuf[DISPLAY_WIDTH] = '\0';
      pxscroll = 0;
   }
   else
   {
      // init scroll buffer
      memcpy(pdisplaybuf, playbuf, DISPLAY_WIDTH);
      pdisplaybuf[DISPLAY_WIDTH] = '\0';
      pxscroll = 1;
   }

   if (lcd_version == 0)
   {
      pxwrap = 23;
      lcd_update_3rd_line(0);
   }
   else
   {
      if (pxscroll == 0)
      {
	 // scrolling off and force display of new string
	 print("\\K2\"0");
	 lcd_update_3rd_line(1);
      }
      else
      {
	 // just fill scrolling buffer and let the display do the work
	 printf("\\[\\K3\"1%s    \\K3\"0\\K2\"1\\]", playbuf);
      }
   }
}

//
//	lcd_update_3rd_line
//
//	Called when timer 0 expires.
//	Displays pdisplaybuf on the third line of the display, then
//	scrolls pdisplaybuf to the left, and grabs the next
//	character from playbuf.
//
void lcd_update_3rd_line(const unsigned char refresh)
{
   xdata unsigned char font;

   if (current_control == CONTROL4)
      font = 33;  // bold
   else
      font = 32;  // normal
	
   if (lcd_version == 0)
   {
      if (pxscroll == 0)
      {
	 // not scrolling so just display it
	 printf("\\[\\B \"\\C%c%s\\C \\]\r\n", (char) font, playbuf);
      }
      else
      {
	 printf("\\[\\B \"\\C%c%s\\C \\]\r\n", (char) font, pdisplaybuf);

	 // shift buffer & insert new char
	 pxwrap++;
	 memcpy(pdisplaybuf, pdisplaybuf+1, DISPLAY_WIDTH - 1);

	 if(pxwrap < playbuf_len) {
	    pdisplaybuf[DISPLAY_WIDTH-1] = playbuf[pxwrap];
	 } else if (pxwrap < playbuf_len + 4) {
	    pdisplaybuf[DISPLAY_WIDTH-1] = ' ';
	 } else {
	    pxwrap = 0;
	    pdisplaybuf[DISPLAY_WIDTH-1] = playbuf[0];
	 }
      }
   }
   else
   {
      // ensure font is correct upon refresh (selection might have changed)
      if (refresh != 0)
      {
	 if (pxscroll != 0) 
	 {
	    printf("\\K1\"%c", (char) font);
	 }
	 else
	 {
	    printf("\\[\\B \"\\C%c%s\\C \\]\r\n", (char) font, playbuf);
	 }
      }
   }
}


void lcd_display_id3_init()
{
        // reset id3 data at the start of each new track and when no id3 data present

	// memset is highly optimized for initializing memory (even
	// faster than declaring an initialized variable with SDCC,
	// and it uses much less space... so please try to use memset
	// when you need to fill arrays or structs with an initial byte
	memset(&id3_tag, 0, sizeof(id3_tag));

	// force display update
	lcd_screen_dynamic(1);
}

void lcd_display_id3_struct() {

   id3_tag.title[DISPLAY_WIDTH] = 0;
   id3_tag.artist[DISPLAY_WIDTH] = 0;
   id3_tag.album[DISPLAY_WIDTH] = 0;
   id3_tag.comment[DISPLAY_WIDTH] = 0;


   // Display ID3 data
   // use display clear line functions with version 1 displays
   printf("\\L#\\L$\\L%%\\[\\B #%s\\B $%s\\B %%%s\\]\r\n", 
	  id3_tag.title, 
	  id3_tag.artist, 
	  id3_tag.album);
}

void lcd_update_time() {
	xdata unsigned char s[6];
	xdata unsigned char min;
	xdata unsigned int sec;	
	// Now get the elapsed time to display.  Subtract the number of frames played when
	// this song started (updated in main.c when the song changes) from the current number
	// of frames played back.  Multiply that by the number of milliseconds per from for the 
	// current sample rate (defined in the playback struct in display.c).  Then divide the whole
	// mess by 1000 to get seconds.  Break it up to seconds & minutes, then output.
	// -ZSB 12-Aug-2001 3:33 PM
		
	sec = ((GetFrameCount() - songStartFrameCount) * playback.ms_per_frame) / 1000;
	min = sec / 60;
	sec = sec % 60;

	if(min < 10) {
		s[0] = '0';
	} else {
		s[0] = (min / 10) + '0';
	}
	
	s[1] = (min % 10) + '0';

	s[2] = ':';

	if(sec < 10) {
		s[3] = '0';
	} else {
		s[3] = (sec / 10) + '0';
	}
	
	s[4] = (sec % 10) + '0';

	s[5] = 0;

	printf("\\[\\B3 %s\\]\r\n",s);
}








void lcd_update_top_line(void) {	
	if (playing) {
		lcd_navigation(0,0,"Playing ",CONTROL1);
	} else {
		lcd_navigation(0,0,"Paused  ",CONTROL1);
	}

	lcd_navigation(8, 0, play_mode_description[play_mode], CONTROL2);
}





void lcd_volume_set(unsigned char attn)
//
//	lcd_volume_set
//
//	Displays a volume bargraph on the bottom line of
//	the display.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(0x40-attn)/4;

	print("\\[\\B 'Vol: [");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			print("\\C!\\w\\C ");
		}
		else
		{
			print("-");
		}
	}
	print("]\\]");
}










void lcd_bass_set(char bass)
//
//	lcd_bass_set
//
//	Displays a bass bargraph on the bottom line of
//	the display.
//
//	The sta013 expects a 2s compliment number from
//	-12 to 12 corresponding to a bass boost of
//	-18dB to 18dB.
//
{
	unsigned char x,z;

	// Make the bargraph fit in 17 characters

	x=(bass+12)*17/25;
	lcd_navigation(0,5,"Bas: ",CONTROL7);

	print("\\[\\B%%[");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			print("\\C!\\w\\C ");
		}
		else
		{
			print("-");
		}
	}
	print("]\\]\r\n");
}





void lcd_treble_set(char treble)
//
//	lcd_treble_set
//
//	Displays a treble bargraph on the bottom line of
//	the display.
//
//	The sta013 expects a 2s compliment number from
//	-12 to 12 corresponding to a treble boost of
//	-18dB to 18dB.
//
{
	unsigned char z,x;

	// Make the bargraph fit in 17 characters
	x=(treble+12)*17/25;

	lcd_navigation(0,3,"Tre: ",CONTROL5);

	print("\\[\\B%#[");

	for(z=0;z<17;z++)
	{
		if(z==x)
		{
			// Character 151 in font 1
			print("\\C!\\w\\C ");
		}
		else
		{
			print("-");
		}
	}
	print("]\\]\r\n");
}





void lcd_bass_freq_set(unsigned int bass_frequency)
//
//	lcd_bass_freq_set
//
//	Displays a bass frequency on the display.
//
{
	lcd_navigation(0,6,"Bass Freq:   ", CONTROL8);
	printf("\\[\\B-&%4d Hz\\]\r\n", bass_frequency);
}



void lcd_volume_set_db(unsigned char attn,unsigned char x, unsigned char y)
//
//	lcd_volume_set_db
//
//	Displays a volume db at the specified coordinates.
//
{
	lcd_navigation(x,y,"Vol: ",CONTROL0);

	if(attn==0)
	{
		print("\\[\\B%'  0 dB\\]\r\n");
	}
	else
	{
		printf("\\[\\B%%'-%2d dB\\]\r\n",attn);
	}
}


void lcd_treble_freq_set(unsigned int treble_frequency)
//
//	lcd_treble_freq_set
//
//	Displays a treble frequency on the display.
//
{
	lcd_navigation(0,4,"Treble Freq: ", CONTROL6);
	printf("\\[\\B-$%4d Hz\\]\r\n", treble_frequency);
}





void lcd_clear_bottom_row(void)
//
//	lcd_clear_bottom_row
//
//	Erases all text on the bottom row of the
//	display.  Set current button to nothing.
//
{
	print("\\[\\B '                        \\]");
}





void lcd_change_menu(unsigned char action)
//
//	lcd_change_menu
//
//	Clears display and displays the initial data
//	for the current screen.  Increments current screen
//	if action==1;
//
{
	// Some sanity checks
	if(action==1)
	{
		if((++current_screen) > SCREEN_MAX)
			current_screen=SCREEN1;
		if (current_control > last_control[current_screen])
		   current_control = CONTROL0;
	}

	if (lcd_version != 0)
	{
	   // display updates off
	   print("\\I:");

	   // clear lines that change only
	   print("\\L#\\L$\\L%\\L&");
	}
	else
	{
	   print("\\[\\@ 0\\]\r\n");	// Clear display
	}

	switch(current_screen)
	{
	  case SCREEN1:
		lcd_update_top_line();
		lcd_update_2nd_line(1);
		lcd_update_3rd_line(1);
		lcd_screen_dynamic(1);
		break;

	  case SCREEN2:
		lcd_update_top_line();
		lcd_update_2nd_line(1);
		lcd_update_3rd_line(1);
		lcd_screen_dynamic(1);
		break;

	  case SCREEN3:
		lcd_update_top_line();
		lcd_update_2nd_line(1);
		lcd_update_3rd_line(1);
		lcd_screen_dynamic(1);
 		break;

	  default:
		break;
	}

	if (lcd_version != 0)
	{
	   // redraw whole display & display updates on
	   print("\\I8\\I9");
	}
}





void lcd_navigation(unsigned char x, unsigned char y,
	const unsigned char *navtext, unsigned char this_control)
//
//	lcd_navigation
//
//	Display navigation element on the screen at coordinates (x,y).
//	If this_control is the active one then display in bold.
//
{
	if(this_control==current_control)
	{
		printf("\\[\\B%c%c\\C!%s\\C \\]\r\n",(char)(x+32),(char)(y+32),navtext);
	}
	else
	{
		printf("\\[\\B%c%c\\C %s\\C \\]\r\n",(char)(x+32),(char)(y+32),navtext);
	}
}


void lcd_setup(void)
{
	print("lcd:");

	parse_event_setup();

	// setup navigation
	current_screen = SCREEN1;
	current_control = CONTROL0;

	// ask display for version
	lcd_version = 0;
	print("\\E\r\n");

	string_init(filebuf, ' ', FILEBUF_MAX);
	string_init(fdisplaybuf, ' ', DISPLAY_WIDTH + 1);
	string_init(playbuf, ' ', FILEBUF_MAX);
	string_init(pdisplaybuf, ' ', DISPLAY_WIDTH + 1);

	pxscroll = 0;
	playbuf_len = 0;
	pxwrap = 0;

	fxscroll = 0;
	filebuf_len = 0;
	fxwrap = 0;

//	param_display_init();
}

void lcd_screen_dynamic(const unsigned char refresh)
//
//	lcd_screen_dynamic
//
//	Update dynamic data.
//
{

   static xdata unsigned char last_bitrate = 0;
   static xdata unsigned char last_freq = 0;
   static xdata unsigned char last_mode = 0;

	switch(current_screen)
	{
	  case SCREEN1:
		// Now this bit should only update the screen if it "has to."  IE if an item has changed or if
		// we're doing a screen refresh. -ZSB 18-Aug-2001 2:29 PM
		playback.mode_known = 0; // force update_playback_params to reget the info for the new bitrate (if applicable)
		update_playback_parms(); // This should get any new bitrate for VBR's
		// Is there any way to tell if an MP3 is VBR or not?  If so, we could skip calling this for CBR files.

		if (playback.mode_known) {
		        // On a refresh or change, we send out the whole thing
			if(refresh
			   || last_bitrate != playback.bitrate
			   || last_freq != playback.freq
			   || last_mode != playback.mode[0]) {
				last_bitrate = playback.bitrate;
				last_mode = playback.mode[0];
				last_freq = playback.freq;
				printf("\\[\\B &%3d kbps  %2d kHz  %s\\]", last_bitrate, playback.freq, playback.mode);
			}
		} else { 
		        // We'll always display blanks, regardless of the
		        // bitrate settings if things are undefined.
			// Just to make sure we clear the screen quickly....
			print("\\[\\B &    kbps     kHz        \\]");
			last_bitrate = 0;
			last_freq = 0;
			last_mode = 0;
		}

		if (refresh) {
		   lcd_display_id3_struct();
		}

		// Update the running time every call (need to do this twice a second for a stead seconds display)
		lcd_update_time();
		break;

	  case SCREEN2:
	        if (refresh)
	        {
		   lcd_treble_set(setting.treble);
		   lcd_treble_freq_set(treble_freq_table[setting.treble_freq_index]);
		   lcd_bass_set(setting.bass);
		   lcd_bass_freq_set(bass_freq_table[setting.bass_freq_index]);
		}
		lcd_update_time();
		break;

	  case SCREEN3:
		lcd_update_time();
		break;

	  default:
		break;
	}

	// Should only need this on refreshes.  The change volume
	// function handles updating it when the volume changes.
	if(refresh) 
	{
	   lcd_volume_set_db(setting.attn,0,7);
	}
}



void lcd_change(unsigned char parm)
{
	switch (parm) {

	  case LCD_VOLUME:
		lcd_volume_set_db(setting.attn,0,7);
		return;

	  case LCD_TREBLE:
		lcd_treble_set(setting.treble);
		return;

	  case LCD_BASS:
		lcd_bass_set(setting.bass);
		return;

	  case LCD_PLAY_MODE:
	  case LCD_RANDOM_MODE:
	        lcd_update_top_line();
	        return;

	  case LCD_TREBLE_FREQ:
		lcd_treble_freq_set(treble_freq_table[setting.treble_freq_index]);
		return;

	  case LCD_BASS_FREQ:
		lcd_bass_freq_set(bass_freq_table[setting.bass_freq_index]);
		return;

	  default:
		printf("Unknown LCD change event: %d\n", parm);
	}
}

/* handle user interface events that related to navigation of */
/* the on-screen information */

// SCREEN 1 ACTIONS

void user_action_screen1(event_t event)
{
   static code up_control[CONTROL_MAX+1] = {
      CONTROL4, CONTROL0, CONTROL0, CONTROL1,
      CONTROL3, CONTROL5, CONTROL6, CONTROL7,
      CONTROL8, CONTROL9};

   static code down_control[CONTROL_MAX+1] = {
      CONTROL1, CONTROL3, CONTROL3, CONTROL4,
      CONTROL0, CONTROL5, CONTROL6, CONTROL7,
      CONTROL8, CONTROL9};

   switch(event) {
      case E_UP:
	 current_control = up_control[current_control];
	 lcd_change_menu(0);
	 break;
      case E_DOWN:
	 current_control = down_control[current_control];
	 lcd_change_menu(0);
	 break;
      case E_INC:
	 switch(current_control) {
	    case CONTROL0:		// Volume
	       action_vol_up();
	       break;
	    case CONTROL1:		// Play/Pause
	       put_back_event(E_PLAY_PAUSE);
	       break;
	    case CONTROL2:		// Normal/Random
	       random_mode_next();
	       lcd_update_top_line();
	       break;
	    case CONTROL3:		// Filename
	       put_back_event(E_NEXT);
	       break;
	    case CONTROL4:		// Directory/Playlist
	       put_back_event(E_NEXT_LIST);
	       break;
	    default:
	       break;
	 }
	 break;
      case E_DEC:
	 switch(current_control) {
	    case CONTROL0:		// Volume
	       action_vol_down();
	       break;
	    case CONTROL1:		// Play/Pause
	       put_back_event(E_PLAY_PAUSE);
	       break;
	    case CONTROL2:		// Normal/Random
	       random_mode_prev();
	       lcd_update_top_line();
	       break;
	    case CONTROL3:		// Filename
	       put_back_event(E_PREV);
	       break;
	    case CONTROL4:		// Directory/Playlist
	       put_back_event(E_PREV_LIST);
	       break;
	    default:
	       break;
	 }
	 break;
      default:
	 printf("Unsupported event %d\r\n", event);
	 break;
   }
}

// SCREEN 2 ACTIONS

void user_action_screen2(event_t event)
{
   static code up_control[CONTROL_MAX+1] = {
      CONTROL8, CONTROL0, CONTROL0, CONTROL1,
      CONTROL3, CONTROL4, CONTROL5, CONTROL6,
      CONTROL7, CONTROL9};

   static code down_control[CONTROL_MAX+1] = {
      CONTROL1, CONTROL3, CONTROL3, CONTROL4,
      CONTROL5, CONTROL6, CONTROL7, CONTROL8,
      CONTROL0, CONTROL9};

   switch(event) {
      case E_UP:
	 current_control = up_control[current_control];
	 lcd_change_menu(0);
	 break;
      case E_DOWN:
	 current_control = down_control[current_control];
	 lcd_change_menu(0);
	 break;
      case E_INC:
	 switch(current_control) {
	    case CONTROL0:		// Volume
	       action_vol_up();
	       break;
	    case CONTROL1:		// Play/Pause
	       put_back_event(E_PLAY_PAUSE);
	       break;
	    case CONTROL2:		// Random Mode
	       random_mode_next();
	       lcd_update_top_line();
	       break;
	    case CONTROL3:		// Filename
	       put_back_event(E_NEXT);
	       break;
	    case CONTROL4:		// Directory/Playlist
	       put_back_event(E_NEXT_LIST);
	       break;
	    case CONTROL5:		// Treble
	       action_treble_up();
	       break;
	    case CONTROL6:		// Treble frequency
	       action_treble_freq_up();
	       break;
	    case CONTROL7:		// Bass
	       action_bass_up();
	       break;
	    case CONTROL8:		// Bass frequency
	       action_bass_freq_up();
	       break;
	    default:
	       break;
	 }
	 break;
      case E_DEC:
	 switch(current_control) {
	    case CONTROL0:		// Volume
	       action_vol_down();
	       break;
	    case CONTROL1:		// Play/Pause
	       put_back_event(E_PLAY_PAUSE);
	       break;
	    case CONTROL2:		// Normal/Random
	       random_mode_prev();
	       lcd_update_top_line();
	       break;
	    case CONTROL3:		// Filename
	       put_back_event(E_PREV);
	       break;
	    case CONTROL4:		// Directory/Playlist
	       put_back_event(E_PREV_LIST);
	       break;
	    case CONTROL5:		// Treble
	       action_treble_down();
	       break;
	    case CONTROL6:		// Treble frequency
	       action_treble_freq_down();
	       break;
	    case CONTROL7:		// Bass
	       action_bass_down();
	       break;
	    case CONTROL8:		// Bass frequency
	       action_bass_freq_down();
	       break;
	    default:
	       current_screen=SCREEN1;
	       break;
	 }
	 break;
      default:
	 printf("Unsupported event %d\r\n", event);
	 break;
   }
}

// SCREEN 3 ACTIONS

void user_action_screen3(event_t event)
{

   switch(event) {
      case E_INC:
	 switch(current_control) {
	    case CONTROL0:		// Volume
	       action_vol_up();
	       break;
	    case CONTROL1:		// Play/Pause
	       put_back_event(E_PLAY_PAUSE);
	       break;
	    case CONTROL2:		// Normal/Random
	       random_mode_next();
	       lcd_update_top_line();
	       break;
	    case CONTROL3:		// Filename
	       put_back_event(E_NEXT);
	       break;
	    case CONTROL4:		// Directory/Playlist
	       put_back_event(E_NEXT_LIST);
	       break;
	    default:
	       break;
	 }
	 break;
      case E_DEC:
	 switch(current_control) {
	    case CONTROL0:		// Volume
	       action_vol_down();
	       break;
	    case CONTROL1:		// Play/Pause
	       put_back_event(E_PLAY_PAUSE);
	       break;
	    case CONTROL2:		// Normal/Random
	       random_mode_prev();
	       lcd_update_top_line();
	       break;
	    case CONTROL3:		// Filename
	       put_back_event(E_PREV);
	       break;
	    case CONTROL4:		// Directory/Playlist
	       put_back_event(E_PREV_LIST);
	       break;
	    default:
	       break;
	 }
	 break;
      default:
	 printf("Unsupported event %d\r\n", event);
	 break;
   }
}	

void lcd_user_action(event_t event)
{
   set_timer(1,SCREEN_TIMEOUT);
   switch(event) {
      case E_DISPLAY1:
	 lcd_version = 1;
	 printf("\r\nDisplay protocol version=%d.\r\n", lcd_version);
	 // set scrolling speed (must be same for all lines)
	 print("\\K04");
	 break;

      case E_ENTER:
	 lcd_change_menu(1);            // Enter always changes pages
	 break;

      case E_RIGHT:
	 if (current_control == last_control[current_screen]) {
	    current_control = CONTROL0;
	 } else {
	    current_control++;
	 }
	 lcd_change_menu(0);
	 break;

      case E_LEFT:
	 if (current_control == CONTROL0) {
	    current_control = last_control[current_screen];
	 } else {
	    current_control--;
	 }
	 lcd_change_menu(0);
	 break;

      case E_TIMER1:
	 if(current_screen!=SCREEN1 || 
	    current_control!=CONTROL0) {
	    current_screen=SCREEN1;
	    current_control=CONTROL0;
	    lcd_change_menu(0);
	 }
	 set_timer(1,SCREEN_TIMEOUT);
	 break;

      default:
	 // all other events are screen specific and sent to a particular screen
	 switch(current_screen) {
	    case SCREEN1:
	       user_action_screen1(event);
	       break;
	    case SCREEN2:
	       user_action_screen2(event);
	       break;
	    case SCREEN3:
	       user_action_screen3(event);
	       break;
	    default:
	       current_screen=SCREEN1;
	       break;
	 }
	 break;
   }
}



